#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include "avwio.h"

AVW *AvwOpen(const char *filename, const char *opts)
{
  AVW *avw;
  char imgfname[1024], hdrfname[1024];
  
  if((avw = (AVW *) calloc(1, sizeof(AVW))) == NULL)
    {
      perror("AvwOpen: Error while reserving memory for AVW!");
      return NULL;
    }

  sprintf(imgfname, "%s.img", filename);
  sprintf(hdrfname, "%s.hdr", filename);

  if((avw->hdrfp = fopen(hdrfname, opts)) == NULL)
    {
      perror("AvwOpen: Error while opening header!");
      free(avw);
      return NULL;
    }
  
  if((avw->imgfp = fopen(imgfname, opts)) == NULL)
    {
      perror("AvwOpen: Error while opening image!");
      free(avw);
      return NULL;
    }

  if(opts[0] == 'r')
    {
      if(!fread(&avw->header, sizeof(struct dsr), 1, avw->hdrfp)) 
	{
	  perror("AvwOpen: Error while reading header!");
	  return NULL;
	}

      avw->mode = AVW_READ;
    }
  else
    {
      avw->mode = AVW_WRITE;
      AvwInitHeader(avw, DT_SIGNED_SHORT, 
		    1, 1, 1, 0,
		    0.0, 0.0, 0.0, 0.0,
		    4,
		    "mm");
    }

  if( !(avw->sameorder = (avw->header.hk.sizeof_hdr == 348)) ) 
    {
      AvwReverseDsr(avw);
      assert(avw->header.hk.sizeof_hdr == 348);
    }

  return avw;     
}
