/*  glim.h

    Mark Woolrich, FMRIB Image Analysis Group

    Copyright (C) 1999-2000 University of Oxford  */

/*  Part of FSL - FMRIB's Software Library
    WWW:      http://www.fmrib.ox.ac.uk/fsl
    Email:    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.
    
    This program is distributed in the hope that it will be useful, but in
    order that the University as a charitable foundation protects its
    assets for the benefit of its educational and research purposes, the
    University makes clear that no condition is made or to be implied, nor
    is any warranty given or to be implied, as to the accuracy of FSL, or
    that it will be suitable for any particular purpose or for use under
    any specific conditions.  Furthermore, the University disclaims all
    responsibility for the use which is made of FSL.  See the GNU General
    Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA */

#include <iostream>
#include <fstream>
#define WANT_STREAM
#define WANT_MATH

#include "newmatap.h"
#include "newmatio.h"
#include "VolumeSeries.h"
#include "Volume.h"

#ifndef NO_NAMESPACE
using namespace NEWMAT;
using namespace TACO;
namespace SIGPROC{
#endif

#if !defined(__glim_h)
#define __glim_h
 
#define FALSE 0
#define TRUE 1

#define BATCHSIZE 50

  class Glim
    {
    public:
      Glim(const VolumeSeries& p_y, const Matrix& p_x); 

      void Save();

      const VolumeSeries& ComputeResids();      
      void ComputePes();
      void SetVrow(const ColumnVector& p_vrow, const int ind);
      void SetGlobalVrow(const ColumnVector& p_vrow);
      void ComputeSigmaSquared(const int ind);
      void UseGlobalVrow();

    private:
      Glim();
      Glim(const Glim&);
      Glim& operator=(const Glim& p_glim);
      
      void SetCorrection(const Matrix& corr, const int ind);
      void ConstructV(const ColumnVector& p_vrow);

      // y = bx + r
      const VolumeSeries& y;
      const Matrix& x;

      int numTS;
      int sizeTS;
      int numParams;

      VolumeSeries r;
      Matrix pinv_x;
      Matrix V;
      Matrix RV;
      Matrix RMat;

      int batch_size;

      // Data to be saved:
      Matrix corrections;
      Matrix b;
      Volume sigmaSquareds;
      float dof;
    };


#ifndef NO_NAMESPACE
       }
#endif

#endif




