/*  interpolation.h

    Mark Jenkinson, FMRIB Image Analysis Group

    Copyright (C) 1999-2000 University of Oxford  */

/*  Part of FSL - FMRIB's Software Library
    WWW:      http://www.fmrib.ox.ac.uk/fsl
    Email:    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.
    
    This program is distributed in the hope that it will be useful, but in
    order that the University as a charitable foundation protects its
    assets for the benefit of its educational and research purposes, the
    University makes clear that no condition is made or to be implied, nor
    is any warranty given or to be implied, as to the accuracy of FSL, or
    that it will be suitable for any particular purpose or for use under
    any specific conditions.  Furthermore, the University disclaims all
    responsibility for the use which is made of FSL.  See the GNU General
    Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA */

// Interpolation functions


#if !defined(__interpolation_h)
#define __interpolation_h

#include "mjimage.h"

#ifndef NO_NAMESPACE
 using namespace MJIMAGE;

namespace INTERPOLATION {
#endif

  inline float q_tri_interpolation(const volume& v, 
				   const float x, const float y, const float z)
    {
      int ix, iy, iz;
      ix=(int) x; iy=(int) y; iz=(int) z;
      float dx=x-ix, dy=y-iy, dz=z-iz;
      float v000, v001, v010, v011, v100, v101, v110, v111;
      v.getneighbours(ix,iy,iz, v000,v001,v010,v011,v100,v101,v110,v111);
      {
	float temp1, temp2, temp3, temp4, temp5, temp6;
	temp1 = (v100 - v000)*dx + v000;
	temp2 = (v101 - v001)*dx + v001;
	temp3 = (v110 - v010)*dx + v010;
	temp4 = (v111 - v011)*dx + v011;
	// second order terms
	temp5 = (temp3 - temp1)*dy + temp1;
	temp6 = (temp4 - temp2)*dy + temp2;
	// final third order term
	return (temp6 - temp5)*dz + temp5;
      }
    }


  inline float q_tri_interpolation(const volume& v, 
				   const float x, const float y, const float z,
				   const int ix, const int iy, const int iz)
    {
      float dx=x-ix, dy=y-iy, dz=z-iz;
      float v000, v001, v010, v011, v100, v101, v110, v111;
      v.getneighbours(ix,iy,iz, v000,v001,v010,v011,v100,v101,v110,v111);
      {
	float temp1, temp2, temp3, temp4, temp5, temp6;
	temp1 = (v100 - v000)*dx + v000;
	temp2 = (v101 - v001)*dx + v001;
	temp3 = (v110 - v010)*dx + v010;
	temp4 = (v111 - v011)*dx + v011;
	// second order terms
	temp5 = (temp3 - temp1)*dy + temp1;
	temp6 = (temp4 - temp2)*dy + temp2;
	// final third order term
	return (temp6 - temp5)*dz + temp5;
      }
    }


  float tri_interpolation(const volume& v, const float x, const float y,
			  const float z);

  bool filled_bounds(int x, int y, int z, int xb, int yb, int zb);
  float filled_tri_interpolation(const volume& v, const float x, const float y,
			  const float z);

  float bi_interpolation(const volume& v, const float x, const float y);

#ifndef NO_NAMESPACE
 }
#endif

#endif
