#include <cstdlib>
#include <iostream>
#include <string>
#include <strstream>
#include "mjavwio.h"
#include "AvwRead.h"

#ifndef NO_NAMESPACE
 namespace MJAVWIO {
#endif

void AvwRead(AVW* IP, volume& output)
{
    int x,y,z,v,type;
    AvwGetDim(IP,&x,&y,&z,&v);
    size_t imagesize=x*y*z*v;
    AvwGetDataType(IP,&type);

    float* fbuffer=new float[imagesize];

    switch(type)
      {
      case DT_SIGNED_SHORT:
	{
	  short* sbuffer=new short[imagesize];
	  AvwReadVolumes(sbuffer,v,IP);
	  for(size_t i=0; i<imagesize; i++) fbuffer[i]=sbuffer[i];
	  delete[] sbuffer;
	}
	break;
      case DT_UNSIGNED_CHAR:
	{
	  unsigned char* sbuffer=new unsigned char[imagesize];
	  AvwReadVolumes(sbuffer,v,IP);
	  for(size_t i=0; i<imagesize; i++) fbuffer[i]=sbuffer[i];
	  delete[] sbuffer;
	}
	break;
      case DT_SIGNED_INT:
	{
	  cerr << "WARNING: untested read on signed int" << endl;
	  int* sbuffer=new int[imagesize];
	  AvwReadVolumes(sbuffer,v,IP);
	  for(size_t i=0; i<imagesize; i++) fbuffer[i]=sbuffer[i];
	  delete[] sbuffer;
	}
	break;
      case DT_DOUBLE:
	{
	  cerr << "WARNING: untested read on double" << endl;
	  double* sbuffer=new double[imagesize];
	  AvwReadVolumes(sbuffer,v,IP);
	  for(size_t i=0; i<imagesize; i++) fbuffer[i]=sbuffer[i];
	  delete[] sbuffer;
	}
	break;
      case DT_FLOAT:
	AvwReadVolumes(fbuffer,v,IP);
	break;
      default:
	ostrstream errmsg;
	errmsg << "Avwread: DT " << type <<  " not supported";
	perror(errmsg.str());
      }

    output.reinitialize(x,y,z,fbuffer);

    return;
}

void AvwReadSingle(AVW* IP, volume& output)
{
  int x,y,z,v,type;
  AvwGetDim(IP,&x,&y,&z,&v);
  size_t imagesize=x*y*z;
  AvwGetDataType(IP,&type);
  
  float* fbuffer=new float[imagesize];
  
  switch(type)
    {
    case DT_SIGNED_SHORT:
      {
	short* sbuffer=new short[imagesize];
	AvwReadVolumes(sbuffer,1,IP);
	for(size_t i=0; i<imagesize; i++) fbuffer[i]=sbuffer[i];
	delete[] sbuffer;
      }
      break;
    case DT_UNSIGNED_CHAR:
      {
	unsigned char* sbuffer=new unsigned char[imagesize];
	AvwReadVolumes(sbuffer,1,IP);
	for(size_t i=0; i<imagesize; i++) fbuffer[i]=sbuffer[i];
	delete[] sbuffer;
      }
      break;
    case DT_SIGNED_INT:
      {
	cerr << "WARNING: untested read on signed int" << endl;
	int* sbuffer=new int[imagesize];
	AvwReadVolumes(sbuffer,1,IP);
	for(size_t i=0; i<imagesize; i++) fbuffer[i]=sbuffer[i];
	delete[] sbuffer;
      }
      break;
    case DT_DOUBLE:
      {
	cerr << "WARNING: untested read on double" << endl;
	double* sbuffer=new double[imagesize];
	AvwReadVolumes(sbuffer,1,IP);
	for(size_t i=0; i<imagesize; i++) fbuffer[i]=sbuffer[i];
	delete[] sbuffer;
      }
      break;
    case DT_FLOAT:
      AvwReadVolumes(fbuffer,1,IP);
      break;
    default:
      ostrstream errmsg;
      errmsg << "Avwread: DT " << type <<  " not supported";
      perror(errmsg.str());
    }
  
  output.reinitialize(x,y,z,fbuffer);
  
  return;
}

void AvwWrite(AVW* OP, const volume& outputvol)
{
  unsigned int zb = outputvol.slices(), yb = outputvol.rows(), 
    xb = outputvol.columns();
  float *qv = new float[xb];

  AvwSetDim(OP,outputvol.columns(),outputvol.rows(),outputvol.slices(),1);
  AvwSetDataType(OP, DT_FLOAT);
  if (OP->voxelbytes == 0)  
    OP->voxelbytes = sizeof(float);

  for (unsigned int z=0; z<zb; z++) { 
    for (unsigned int y=0; y<yb; y++) { 
      for (unsigned int x=0; x<xb; x++) { 
	qv[x] = outputvol(x,y,z);
      }
      AvwWriteBuffer(qv,xb,OP);
      // fwrite(qv,xb*sizeof(float),1,OP->imgfp);
    }
  }
  delete [] qv;
  AvwWriteHeader(OP);
  // fwrite(&OP->header,sizeof(OP->header),1,OP->hdrfp);
}

#ifndef NO_NAMESPACE
 }
#endif
