#

#{{{ copyright etc.

#   susan_proc.tcl - TCL call for susan smoothing
#
#   Stephen Smith, FMRIB Image Analysis Group
#
#   Copyright (C) 1999-2000 University of Oxford
#
#   Part of FSL - FMRIB's Software Library
#   WWW:      http://www.fmrib.ox.ac.uk/fsl
#   Email:    fsl@fmrib.ox.ac.uk
#   
#   Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
#   Imaging of the Brain), Department of Clinical Neurology, Oxford
#   University, Oxford, UK
#   
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or (at
#   your option) any later version.
#   
#   This program is distributed in the hope that it will be useful, but in
#   order that the University as a charitable foundation protects its
#   assets for the benefit of its educational and research purposes, the
#   University makes clear that no condition is made or to be implied, nor
#   is any warranty given or to be implied, as to the accuracy of FSL, or
#   that it will be suitable for any particular purpose or for use under
#   any specific conditions.  Furthermore, the University disclaims all
#   responsibility for the use which is made of FSL.  See the GNU General
#   Public License for more details.
#   
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#   USA

#}}}

proc susan_proc { Folder Volume bt dt dim um usans usanlist } {
    #{{{ setup vars

global PXHOME FSLDIR USER PROCID

#}}}
    #{{{ test type

    set type [ what_type 0 ]
    if { $type < 2 || $type > 3 } {
        puts "Selected page isn't a 2D/3D image!"
        MxPause "Selected page isn't a 2D/3D image!"
        return 1
    }

#}}}
    #{{{ get image props, test, save primary SUSAN input and initialize thecommand

    MxGetImageProperties $Volume pl

    set bpp [keylget pl Bits]
    set enc [keylget pl Encoding]
    set name [keylget pl Name]

    if { $bpp != 16 || $enc != "SI" } {
	puts "SUSAN can only process 16 bit signed data"
	MxPause "SUSAN can only process 16 bit signed data"
	return
    }

    set TF "/tmp/${USER}_${PROCID}_susan"
    
    MxSaveImageAs $Volume AVW ${TF}.hdr true

    set thecommand "exec $FSLDIR/bin/susan_smooth ${TF} $bt ${TF}_out $dt $dim $um $usans"

#}}}
    #{{{ prepare thecommand if separate usans are used

    set i 1
    while { $i <= $usans } {
	set Usans($i) [ lindex $usanlist [ expr ( $i - 1 ) * 2 ] ]
	MxSaveImageAs $Usans($i) AVW ${TF}_usan${i}.hdr true
	set bt [ lindex $usanlist [ expr ( ( $i - 1 ) * 2 ) + 1 ] ]
	set thecommand "${thecommand} ${TF}_usan${i} $bt"
	incr i 1
    }

    if { $usans > 0 } {
	set thecommand "${thecommand} ${TF}_usan_size"
    }

#}}}

    ScriptUpdate "Calling $thecommand: As this process is not fully integrated into the heart of MEDx, the traffic lights will not show up red during processing. Wait for this popup to disappear before continuing."
    puts $thecommand
    set result [ catch { eval $thecommand } ErrMsg ]

    if {$result != 0} {
	puts "$ErrMsg"
    }
    CancelScriptUpdate

    if { $usans > 0 } {
        MxOpenImage $Folder ${TF}_usan_size.hdr US
        MxGetImageProperties $US USProps
        keylset USProps Name "USAN size from ${name}"
        MxSetImageProperties $US $USProps
	MxComputeDisplayRange $US umin umax
    }

    MxOpenImage $Folder ${TF}_out.hdr SUSAN
    MxGetImageProperties $SUSAN SUSANProps
    keylset SUSANProps Name "SUSAN smoothed ${name}"
    MxSetImageProperties $SUSAN $SUSANProps
    MxComputeDisplayRange $SUSAN umin umax
}
